from consts import *
import graphlab as gl
import graphlab.aggregate as agg


class PapersAnalyzer(object):
    def __init__(self):
        self._papers_sf = gl.load_sframe(PAPERS_SFRAME)

    def papers_add_features(self):
        self._add_authors_features()
        self._add_references_count()
        self._add_title_tf_ids()
        self._add_keywords()

    def _add_authors_features(self):
        logger.info("Adding authors info")
        authors_sf = gl.load_sframe(PAPER_AUTHOR_AFFILIATIONS_SFRAME)
        authors_sf = authors_sf["Paper ID", "Author ID", "Author sequence number"]
        authors_sf['Author_Seq'] = authors_sf.apply(lambda r: [r["Author ID"], r["Author sequence number"]])
        g = authors_sf.groupby("Paper ID", {"Authors List": agg.CONCAT('Author_Seq')})
        g['Authors List Sorted'] = g["Authors List"].apply(lambda l: sorted(l, key=lambda i: i[1]))
        g['Authors List Sorted'] = g['Authors List Sorted'].apply(lambda l: [i[0] for i in l])
        g.remove_column("Authors List")
        g = g["Paper ID", 'Authors List Sorted']

        self._papers_sf = self._papers_sf.join(g, how="left")
        self._papers_sf['Authors Number'] = self._papers_sf['Authors List Sorted'].apply(lambda l: len(l))

    def _add_references_count(self):
        logger.info("Adding references info")
        papers_refrences_sf = gl.load_sframe(PAPER_REFERENCES_SFRAME)
        ref_count_sf = papers_refrences_sf.groupby("Paper ID", {"References Count": agg.COUNT()})
        self._papers_sf = self._papers_sf.join(ref_count_sf, on="Paper ID", how="left")

    def _add_title_tf_ids(self):
        logger.info("Adding title TF-IDS")
        self._papers_sf["Title Tf-Idf"] = self._papers_sf["Original paper title"].apply(lambda t: t.lower())
        self._papers_sf['Title Tf-Idf'] = gl.text_analytics.tf_idf(self._papers_sf["Title Tf-Idf"])
        self._papers_sf['Title Tf-Idf'] = self._papers_sf['Title Tf-Idf'].dict_trim_by_keys(gl.text_analytics.stopwords(), True)

    def _add_keywords(self):
        logger.info("Adding keywords")
        k_sf = gl.load_sframe(PAPER_KEYWORDS_SFRAME)
        g = k_sf.groupby("Paper ID", {"Keywords List": gl.aggregate.CONCAT("Keyword name")})
        self._papers_sf = self._papers_sf.join(g, how="left")

    def filter_papers(self, min_reference_count=5):
        sf = self._papers_sf[self._papers_sf["References Count"] >= min_reference_count]
        sf = sf[sf['Paper Document Object Identifier (DOI)'] != '']
        return sf
